/*******************************************************************************
* YORKTOWN ADVANCED SGML PARSER (YASP)                                         *
*         An SGML Parser conforming to International Standard ISO 8879 --      *
*         Standard Generalized Markup Language                                 *
*------------------------------------------------------------------------------*
* Written 1987 - 1993 by                                                       *
*         Pierre G. Richard                                                    *
*         IBM T.J. Watson Research Center                                      *
*         Yorktown Heights, New York                                           *
*         petrus@watson.ibm.com                                                *
*------------------------------------------------------------------------------*
* FUNCTION NAME    = YASP / CSD compiler                                       *
* STATUS           = Version 0, Release 1, Level 33 and up                     *
* COMPONENT NAME   = ocschar.c                                                 *
* DESCRIPTIVE NAME = Build a UCTAB and a GTYPE table                           *
* DATE_WR          = 07/02/1987                                                *
* AUTHOR           = Pierre G. Richard - PETRUS at YKTVMX                      *
*                                                                              *
* FUNCTIONAL DESCRIPTION =                                                     *
* This module groups all that relates to the Character Sets:                   *
* - DOCUMENT Character Set Definition (BASESET and DESCSET)                    *
* - Concrete Syntax:                                                           *
*    - SHUNCHAR                                                                *
*    - SYNTAX_REFERENCE Character Set Definition (BASESET and DESCSET)         *
*                                                                              *
* IMPLEMENTATION NOTE =                                                        *
*                                                                              *
* DEPENDENCIES =  (see Include list)                                           *
*                                                                              *
* IMPORT=                                                                      *
*                                                                              *
* EXPORT=                                                                      *
*                                                                              *
* CHANGE HISTORY =                                                             *
* Number Date     Inits    Change & Reason (brief)                             *
*      1 09/22/91 PETRUS   UNUSED in the baseset is like UNUSED keyword        *
*      2 12/08/91 PETRUS   FIND is issued for BASESETS                         *
*      3 12/10/91 PETRUS   Added codepages 850 and 395                         *
*      4 03/31/95 ESPERT   Added ECMA character sets                           *
*******************************************************************************/

/*-------------+
|Include Files |
+-------------*/
#include "machine.h"
#include "portable.h"
#include "yaspapi.h"
#include "global.h"
#include "usrlib.h"
#include "sgml.h"
#include "gtype.h"
#include "ocs.h"

typedef struct t_CHRDSC {
   ushort chtype;
   uchar  s_chnb;
   uchar  upper;     /* system character set # of the upper case form */
}CHRDSC;

/*---------------+
|Local Functions |
+---------------*/
static CHOICES const * _loadds LP_FCT adr_shun_choices(void);
static uchar   const * _loadds LP_FCT adr_minset(void);
static CHRDSC  const * _loadds LP_FCT find_baseset(int);

#define BSEQ_CHAR 'B'
#define CH_CONTROL     S_SHUNNABLE | S_CONTROLS  /* controls */
#define CH_USED        S_SHUNNABLE               /* graphic - mindata */
#define CH_UNUSED      S_NONSGML

/*------------------------------------------------------+
|When M_p->syn_dset is used, M_p->doc_dset is useless:  |
|the gtype and uctab have been built.                   |
|so we can reuse the same room.                         |
|To make the code clear, M_p->doc_dset is a macro.      |
+------------------------------------------------------*/
#define M_P_DOC_DSET M_p->syn_dset


/*STATIC------------------------------------------------------------adr_minset-+
| Allow temporary access to local constant area (mindata, letters, digits)     |
| The 26 first characters must be the lower-case letters.                      |
+-----------------------------------------------------------------------------*/
static uchar const * _loadds LP_FCT adr_minset(void)
{
   static uchar const minset[] =
   "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'()+,-./:=?";
   return(minset);
}

/*STATIC------------------------------------------------------adr_shun_choices-+
| Allow temporary access to local constant area (choices)                      |
+-----------------------------------------------------------------------------*/
static CHOICES const * _loadds LP_FCT adr_shun_choices(void)
{
   static CHOICES const shun_choices[] = {
      { offsetof(ALWDTAB,CONTROLS), CONTROLS_TK },
      { offsetof(ALWDTAB,NONE),     NONE_TK },
      { -1,                         NOT_FOUND_TK }
   };
   return(shun_choices);
}

/*--------------------------------------------------------------------chr_shun-+
|Enter here when SHUNNED has just been decoded.                                |
|Update gtype for shunned characters, finish gtype and uctab.                  |
+-----------------------------------------------------------------------------*/
void P_FCT chr_shun(COMMON_P M_p)
{
   GA_P gbla_p = M_p->gbla_p;
   uint d_chnb;
   int i;
   ushort *gtype_p;
   uchar  *uctab_p;
   DSET     *dset_p;
   boolean is_shunned_controls = FALSE;

   skip_Wps(gbla_p);
   switch(choos_in(gbla_p,adr_shun_choices(),(GNWD *)GA(alwdtab_p))) {
   case NONE_TK:
      skip_Wps(gbla_p);
      break;
   case CONTROLS_TK:
      is_shunned_controls = TRUE;
      skip_Wps(gbla_p);
      /* fall thru */
   default:  /* NOT_FOUND_TK */
      if (!is_digit(GA(curr_char))) {
         if (is_shunned_controls) {
            break;
         }else {
            errorh(gbla_p,NODIG,ERH_T_SEV+ERH_LOCATE);
         }
      }
      do {
         if ((get_chnb(gbla_p, &d_chnb)) &&
            (M_p->gtype[d_chnb] & S_SHUNNABLE)) {
            M_p->gtype[d_chnb] = S_SHUNNED;
         }
         skip_Wps(gbla_p);
      }while (is_digit(GA(curr_char)));
      break;
   }
   gtype_p = M_p->gtype - 1;
   uctab_p = M_p->uctab - 1;
   dset_p = M_P_DOC_DSET - 1;
   i = -1;
   while (++i < MAX_CHARSET) {
      if (!(++dset_p)->is_described) {
         *(++gtype_p) = S_NONSGML;
      }else if (*(++gtype_p) & S_SHUNNABLE) {
         if ((is_shunned_controls) && (*gtype_p & S_CONTROLS)) {
            *gtype_p = S_SHUNNED;
         }else {
            *gtype_p = S_SGML;
         }
      }
      *(++uctab_p) = dset_p->is_based ?
      M_p->charstat[M_p->charstat[dset_p->s_chnb].upper].doc_d_chnb : (uchar)i;
   }
}

/*---------------------------------------------------------------------chr_bld-+
|BASESET / DESCSET parameters                                                  |
|Build an UCTAB and a GTYPE table.                                             |
+-----------------------------------------------------------------------------*/
void P_FCT chr_bld(COMMON_P M_p, boolean is_doc_charset)
{
   GA_P gbla_p = M_p->gbla_p;
   int       d_chnb, b_chnb, num, len;
   uchar     in_doc_or_syn;
   ushort   *gtype_p;
   char      type_of_change;
   EXID      exid;
   ETY       lit_ety;            /* treat literal as a pseudo entity */
   DSET     *dset_p0, *dset_p;
   CHRDSC const *baseset_p, *baseset_p0;
   CHARSTAT *charstat_p, *charstat_p0;
   MSGCODE   no_msgcode, dup_msgcode;
   boolean   is_chnb_ok;

   /*-------------+
   |Initialization|
   +-------------*/
   charstat_p0 = M_p->charstat;
   if (is_doc_charset) {
      no_msgcode = NODRDOC;
      dup_msgcode = DUPDRDOC;
      in_doc_or_syn = IN_DOC_MASK;
      dset_p0 = M_P_DOC_DSET;
   }else {
      no_msgcode = NODRSYN;
      dup_msgcode = DUPDRSYN;
      in_doc_or_syn = IN_SYN_MASK;
      dset_p0 = M_p->syn_dset;
   }
   memset(dset_p0,0,sizeof(M_p->syn_dset));
   chk_gnam(gbla_p,GA(alwdtab_p)->BASESET.s.p);

   do {
      /*------+
      |BASESET|
      +------*/
      skip_ps(gbla_p);
      memset(&exid,0,sizeof(EXID));    /* zero the XID */
      lit_ety.val.lit.len = exid.pid.len = get_lit_min(
         gbla_p, exid.val._.p = lit_ety.val.lit._.p = GA(lit_p)
      );
      exid.val.len = 1 + exid.pid.len;
      lit_ety.is = RCDATA_EF;                 /* tell what it is.. */
      lit_ety.kind = TEXT_EFK;
      lit_ety.name._.p = (char *)0;           /* It's an internal literal */
      exid.name.len = strlen(exid.name._.p = GA(alwdtab_p)->CHARSET.s.p);
      em_open(gbla_p,&lit_ety);               /* re-read it */
      xid_bfpi(gbla_p, &exid);                /* and build the FPI */
      em_close(gbla_p);
      xid_cchk(gbla_p,&exid,CHARSET_XFC);     /* check CHARSET class */
      if ((!em_find(gbla_p, &exid)) ||        /* if Appl can't find it */
         (!(baseset_p0 = find_baseset(exid.idhl.i)))) {  /* or YASP... */
         errorh(gbla_p,INVBASET,ERH_T_SEV+ERH_LOCATE);
         return;  /* avoid compiler warnings - can't return from ERH_T */
      }
      /*----------------------------------------------------------------+
      |DESCSET: Changes the meaning based on the BASESET                |
      +----------------------------------------------------------------*/
      skip_ps(gbla_p);
      chk_gnam(gbla_p,GA(alwdtab_p)->DESCSET.s.p);
      skip_Wps(gbla_p);
      if ((GA(curr_token) != NOT_FOUND_TK)||(!is_digit(GA(curr_char)))) {
         errorh(gbla_p,NODIG,ERH_T_SEV+ERH_LOCATE);
         return;  /* avoid compiler warnings - can't return from ERH_T */
      }
      do {
         is_chnb_ok = get_chnb(gbla_p, (uint *)&d_chnb);
         dset_p = dset_p0 + (--d_chnb);       /* Described Set Char Number */
         gtype_p = M_p->gtype + d_chnb;
         skip_ps(gbla_p);
         num = (uint)get_num(gbla_p) + 1;     /* Number of characters */
         if (!is_chnb_ok) num = 1;            /* if bad chnb, zero it */
         skip_Wps(gbla_p);
         if (GA(curr_token) == NOT_FOUND_TK) {
            if (is_digit(GA(curr_char))) {
               if (!get_chnb(gbla_p, (uint *)&b_chnb)) {
                  num = 1;
               }else {
                  baseset_p = baseset_p0 + (--b_chnb);
                  type_of_change = 1;         /* assigned another meaning */
               }
            }else if (get_gnam(gbla_p,GA(alwdtab_p)->UNUSED.s.p)) {
               type_of_change = 0;            /* UNUSED */
            }else {
               errorh(gbla_p,NUMLITUNU,ERH_T_SEV+ERH_LOCATE);
               return;  /* avoid compiler warnings - can't return from ERH_T */
            }
         }else {
            len = get_lit_min(gbla_p, GA(lit_p));  /* minimum literal */
            type_of_change = 2;
         }
         while (--num) {
            if ((++d_chnb >= MAX_CHARSET)||
               ((type_of_change == 1)&&(++b_chnb > MAX_CHARSET))) {
               errorh(gbla_p,BIGCHDESC,ERH_I_SEV+ERH_LOCATE,d_chnb);
               break;
            }
            ++baseset_p;                /* valid only if type_of_change == 1 */
            ++gtype_p;
            if (!(++dset_p)->is_described) {
               dset_p->is_described = TRUE;
            }else {
               errorh(gbla_p,DUPCHDESC,ERH_W_SEV+ERH_LOCATE,d_chnb);
               continue;
            }
            switch (type_of_change) {
            case 0:               /* UNUSED */
               dset_p->is_unused = TRUE;
               if (is_doc_charset) *gtype_p = S_NONSGML;
               continue;
            case 1:               /* b_chnb assigned */
/*09.22.91*/   if (baseset_p->chtype == CH_UNUSED) {
/*09.22.91*/      dset_p->is_unused = TRUE;
/*09.22.91*/      if (is_doc_charset) *gtype_p = S_NONSGML;
/*09.22.91*/      continue;
/*09.22.91*/   }
               dset_p->is_based = TRUE;
               charstat_p = (dset_p->s_chnb=baseset_p->s_chnb) + charstat_p0;
               if (is_doc_charset) {
                  charstat_p->upper = baseset_p->upper;
                  *gtype_p = baseset_p->chtype;
               }
               break;
            case 2:                               /* Used for something... */
               dset_p->is_parm_lit = TRUE;
               {
                  uchar **c_pp = M_p->parmlit - 1;
                  uint  i = 0;

                  while ((*(++c_pp)) && (strcmp(*c_pp, GA(lit_p)))) {
                     if (++i >= MAX_CHARSET) {
                        errorh(gbla_p,BIGDSCLIT,ERH_T_SEV+ERH_LOCATE);
                        return;  /* avoid warnings - can't return from ERH_T */
                     }
                  }
                  if (!*c_pp) memcpy(*c_pp = malloc(len+1), GA(lit_p), len+1);
                  dset_p->s_chnb = (uchar)i;
                  charstat_p = charstat_p0 + i + MAX_CHARSET;
               }
               if (is_doc_charset) *gtype_p = S_SGML;
               break;
            }
            if (!IS_BITON((charstat_p)->is_described,in_doc_or_syn)) {
               SET_BITON(charstat_p->is_described,in_doc_or_syn);
               if (is_doc_charset) {
                  charstat_p->doc_d_chnb = (uchar)d_chnb;
               }else {
                  charstat_p->syn_d_chnb = (uchar)d_chnb;
               }
            }else {
/*09.22.91*/   errorh(
/*09.22.91*/      gbla_p, DUPCHASGN, ERH_I_SEV+ERH_LOCATE,
/*09.22.91*/      d_chnb, is_doc_charset ?
/*09.22.91*/      charstat_p->doc_d_chnb :
/*09.22.91*/      charstat_p->syn_d_chnb
/*09.22.91*/   );
               SET_BITON(charstat_p->is_multiple,in_doc_or_syn);
            }
         }
         skip_Wps(gbla_p);
      }while ((GA(curr_token) == NOT_FOUND_TK)&&(is_digit(GA(curr_char))));
   }while (get_gnam(gbla_p,GA(alwdtab_p)->BASESET.s.p));

   /*----------------------------------------+
   |Check all LETTERS, DIGIT and Minimum Data|
   |have been described properly             |
   +----------------------------------------*/
   {
      boolean err_found = FALSE;
      uchar const *min_c_p = adr_minset() - 1;
      while (*(++min_c_p)) {
#if (defined(IBMCSET2))          /* IBM C/Set2 bug (01/04/92, V1.0) */
         charstat_p = charstat_p0 + (uint)(*min_c_p);
#else
         charstat_p = charstat_p0 + *min_c_p;
#endif
         if (!IS_BITON(charstat_p->is_described,in_doc_or_syn)) {
            err_found = TRUE;
            errorh(gbla_p,no_msgcode,ERH_W_SEV,*min_c_p);
         }else if (IS_BITON(charstat_p->is_multiple,in_doc_or_syn)) {
            err_found = TRUE;
            errorh(gbla_p,dup_msgcode,ERH_W_SEV,*min_c_p);
         }
      }
      if (err_found) errorh(gbla_p,ABORTED,ERH_T_SEV);
   }
   /*----------------------------------------+
   |After SYNTAX, get rid of the parm_lit:   |
   |we don't need it anymore.                |
   +----------------------------------------*/
   if (!is_doc_charset) {
      uchar **c_pp = M_p->parmlit - 1;
      uint  i = 0;

      while (*(++c_pp)) {
         free(*c_pp);
         if (++i >= MAX_CHARSET) break;
      }
   }else {
      gbla_p->BSEQ_char = (charstat_p0 + BSEQ_CHAR)->doc_d_chnb;
   }
}

/************************* Definition of the Basesets *************************/

/* Minidata */

#define CH_UPP_A       S_LETTER  , 'A'   , 'A'
#define CH_UPP_B       S_LETTER  , 'B'   , 'B'
#define CH_UPP_C       S_LETTER  , 'C'   , 'C'
#define CH_UPP_D       S_LETTER  , 'D'   , 'D'
#define CH_UPP_E       S_LETTER  , 'E'   , 'E'
#define CH_UPP_F       S_LETTER  , 'F'   , 'F'
#define CH_UPP_G       S_LETTER  , 'G'   , 'G'
#define CH_UPP_H       S_LETTER  , 'H'   , 'H'
#define CH_UPP_I       S_LETTER  , 'I'   , 'I'
#define CH_UPP_J       S_LETTER  , 'J'   , 'J'
#define CH_UPP_K       S_LETTER  , 'K'   , 'K'
#define CH_UPP_L       S_LETTER  , 'L'   , 'L'
#define CH_UPP_M       S_LETTER  , 'M'   , 'M'
#define CH_UPP_N       S_LETTER  , 'N'   , 'N'
#define CH_UPP_O       S_LETTER  , 'O'   , 'O'
#define CH_UPP_P       S_LETTER  , 'P'   , 'P'
#define CH_UPP_Q       S_LETTER  , 'Q'   , 'Q'
#define CH_UPP_R       S_LETTER  , 'R'   , 'R'
#define CH_UPP_S       S_LETTER  , 'S'   , 'S'
#define CH_UPP_T       S_LETTER  , 'T'   , 'T'
#define CH_UPP_U       S_LETTER  , 'U'   , 'U'
#define CH_UPP_V       S_LETTER  , 'V'   , 'V'
#define CH_UPP_W       S_LETTER  , 'W'   , 'W'
#define CH_UPP_X       S_LETTER  , 'X'   , 'X'
#define CH_UPP_Y       S_LETTER  , 'Y'   , 'Y'
#define CH_UPP_Z       S_LETTER  , 'Z'   , 'Z'

#define CH_LOW_A       S_LETTER  , 'a'   , 'A'
#define CH_LOW_B       S_LETTER  , 'b'   , 'B'
#define CH_LOW_C       S_LETTER  , 'c'   , 'C'
#define CH_LOW_D       S_LETTER  , 'd'   , 'D'
#define CH_LOW_E       S_LETTER  , 'e'   , 'E'
#define CH_LOW_F       S_LETTER  , 'f'   , 'F'
#define CH_LOW_G       S_LETTER  , 'g'   , 'G'
#define CH_LOW_H       S_LETTER  , 'h'   , 'H'
#define CH_LOW_I       S_LETTER  , 'i'   , 'I'
#define CH_LOW_J       S_LETTER  , 'j'   , 'J'
#define CH_LOW_K       S_LETTER  , 'k'   , 'K'
#define CH_LOW_L       S_LETTER  , 'l'   , 'L'
#define CH_LOW_M       S_LETTER  , 'm'   , 'M'
#define CH_LOW_N       S_LETTER  , 'n'   , 'N'
#define CH_LOW_O       S_LETTER  , 'o'   , 'O'
#define CH_LOW_P       S_LETTER  , 'p'   , 'P'
#define CH_LOW_Q       S_LETTER  , 'q'   , 'Q'
#define CH_LOW_R       S_LETTER  , 'r'   , 'R'
#define CH_LOW_S       S_LETTER  , 's'   , 'S'
#define CH_LOW_T       S_LETTER  , 't'   , 'T'
#define CH_LOW_U       S_LETTER  , 'u'   , 'U'
#define CH_LOW_V       S_LETTER  , 'v'   , 'V'
#define CH_LOW_W       S_LETTER  , 'w'   , 'W'
#define CH_LOW_X       S_LETTER  , 'x'   , 'X'
#define CH_LOW_Y       S_LETTER  , 'y'   , 'Y'
#define CH_LOW_Z       S_LETTER  , 'z'   , 'Z'

#define CH_DIGIT_0     S_DIGIT   , '0'   , '0'
#define CH_DIGIT_1     S_DIGIT   , '1'   , '1'
#define CH_DIGIT_2     S_DIGIT   , '2'   , '2'
#define CH_DIGIT_3     S_DIGIT   , '3'   , '3'
#define CH_DIGIT_4     S_DIGIT   , '4'   , '4'
#define CH_DIGIT_5     S_DIGIT   , '5'   , '5'
#define CH_DIGIT_6     S_DIGIT   , '6'   , '6'
#define CH_DIGIT_7     S_DIGIT   , '7'   , '7'
#define CH_DIGIT_8     S_DIGIT   , '8'   , '8'
#define CH_DIGIT_9     S_DIGIT   , '9'   , '9'

#define CH_SP_QUOTE    S_SPECIAL , '\''  , '\''
#define CH_SP_LPAREN   S_SPECIAL , '('   , '('
#define CH_SP_RPAREN   S_SPECIAL , ')'   , ')'
#define CH_SP_PLUS     S_SPECIAL , '+'   , '+'
#define CH_SP_COMMA    S_SPECIAL , ','   , ','
#define CH_SP_DASH     S_SPECIAL , '-'   , '-'
#define CH_SP_PERIOD   S_SPECIAL , '.'   , '.'
#define CH_SP_SLASH    S_SPECIAL , '/'   , '/'
#define CH_SP_COLON    S_SPECIAL , ':'   , ':'
#define CH_SP_EQUAL    S_SPECIAL , '='   , '='
#define CH_SP_INTER    S_SPECIAL , '?'   , '?'

/*STATIC----------------------------------------------------------find_baseset-+
| Returns a pointer to a CHRDSC structure corresponding to the identifier.     |
| If not found, returns a NULL pointer.                                        |
|                                                                              |
| Each baseset is defined by an unique identifier (parameter literal).         |
| The CHRDSC structure describes, for each code position, the type of          |
| character associated, its translation onto the system character set,         |
| the translation of the upper case form onto the system character set.        |
|                                                                              |
| As the two last values are system-dependent, to allow a greater system       |
| independency of this code:                                                   |
|    - if possible, the character described is entered directly,               |
|      so the translation between system is done automatically                 |
|      when this program is compiled,                                          |
|    - if not, the integer value must be entered.  As it is not the            |
|      same, C pre-processor conditionals must be used:                        |
|      For example, #if (0x20 == ' ') will correspond to ASCII system set.     |
|                                                                              |
| Implementation note:                                                         |
|   - only ASCII or EBCDIC system character set are implemented so far         |
|   - on the 8086, a loadds is done to access to the table,                    |
|     because ds is no more at sp.                                             |
+-----------------------------------------------------------------------------*/
static CHRDSC const * _loadds LP_FCT find_baseset(int ahlid)
{

   /*-------------------------------------------------------------+
   | ISO 646-1983//CHARSET International Reference Version (IRV)  |
   +-------------------------------------------------------------*/
   static CHRDSC ISO_646_1983_chtab[MAX_CHARSET] = {
   /*  00 NUL */  { CH_CONTROL   ,  0x00        ,  0x00        },  /* same */
   /*  01 SOH */  { CH_CONTROL   ,  0x01        ,  0x01        },  /* same */
   /*  02 STX */  { CH_CONTROL   ,  0x02        ,  0x02        },  /* same */
   /*  03 ETX */  { CH_CONTROL   ,  0x03        ,  0x03        },  /* same */
#if (0x20 == ' ')
   /*  04 EOT */  { CH_CONTROL   ,  0x04        ,  0x04        },  /* ASCII */
   /*  05 ENQ */  { CH_CONTROL   ,  0x05        ,  0x05        },  /* ASCII */
   /*  06 ACK */  { CH_CONTROL   ,  0x06        ,  0x06        },  /* ASCII */
#else
   /*  04 EOT */  { CH_CONTROL   ,  0x37        ,  0x37        },  /* EBCDIC */
   /*  05 ENQ */  { CH_CONTROL   ,  0x2D        ,  0x2D        },  /* EBCDIC */
   /*  06 ACK */  { CH_CONTROL   ,  0x2E        ,  0x2E        },  /* EBCDIC */
#endif
   /*  07 BEL */  { CH_CONTROL   ,  '\a'        ,  '\a'        },
   /*  08 BS  */  { CH_CONTROL   ,  '\b'        ,  '\b'        },
   /*  09 HT  */  { CH_CONTROL   ,  '\t'        ,  '\t'        },
   /*  0A LF  */  { CH_CONTROL   ,  '\n'        ,  '\n'        },
   /*  0B VT  */  { CH_CONTROL   ,  '\v'        ,  '\v'        },
   /*  0C FF  */  { CH_CONTROL   ,  '\f'        ,  '\f'        },
   /*  0D CR  */  { CH_CONTROL   ,  '\r'        ,  '\r'        },
   /*  0E SO  */  { CH_CONTROL   ,  0x0E        ,  0x0E        },  /* same */
   /*  0F SI  */  { CH_CONTROL   ,  0x0F        ,  0x0F        },  /* same */
   /*  10 DLE */  { CH_CONTROL   ,  0x10        ,  0x10        },  /* same */
   /*  11 DC1 */  { CH_CONTROL   ,  0x11        ,  0x11        },  /* same */
   /*  12 DC2 */  { CH_CONTROL   ,  0x12        ,  0x12        },  /* same */
   /*  13 DC3 */  { CH_CONTROL   ,  0x13        ,  0x13        },  /* same */
#if (0x20 == ' ')
   /*  14 DC4 */  { CH_CONTROL   ,  0x14        ,  0x14        },  /* ASCII */
   /*  15 NAK */  { CH_CONTROL   ,  0x15        ,  0x15        },  /* ASCII */
   /*  16 SYN */  { CH_CONTROL   ,  0x16        ,  0x16        },  /* ASCII */
   /*  17 ETB */  { CH_CONTROL   ,  0x17        ,  0x17        },  /* ASCII */
#else
   /*  14 DC4 */  { CH_CONTROL   ,  0x3C        ,  0x3C        },  /* EBCDIC */
   /*  15 NAK */  { CH_CONTROL   ,  0x3D        ,  0x3D        },  /* EBCDIC */
   /*  16 SYN */  { CH_CONTROL   ,  0x32        ,  0x32        },  /* EBCDIC */
   /*  17 ETB */  { CH_CONTROL   ,  0x26        ,  0x26        },  /* EBCDIC */
#endif
   /*  18 CAN */  { CH_CONTROL   ,  0x18        ,  0x18        },  /* same */
   /*  19 EM  */  { CH_CONTROL   ,  0x19        ,  0x19        },  /* same */
#if (0x20 == ' ')
   /*  1A SUB */  { CH_CONTROL   ,  0x1A        ,  0x1A        },  /* ASCII */
   /*  1B ESC */  { CH_CONTROL   ,  0x1B        ,  0x1B        },  /* ASCII */
   /*  1C FS  */  { CH_CONTROL   ,  0x1C        ,  0x1C        },  /* ASCII */
#else
   /*  1A SUB */  { CH_CONTROL   ,  0x3F        ,  0x3F        },  /* EBCDIC */
   /*  1B ESC */  { CH_CONTROL   ,  0x27        ,  0x27        },  /* EBCDIC */
   /*  1C FS  */  { CH_CONTROL   ,  0x22        ,  0x22        },  /* EBCDIC */
#endif
   /*  1D GS  */  { CH_CONTROL   ,  0x1D        ,  0x1D        },  /* same ? */
   /*  1E RS  */  { CH_CONTROL   ,  0x1E        ,  0x1E        },  /* same ? */
   /*  1F US  */  { CH_CONTROL   ,  0x1F        ,  0x1F        },  /* same ? */
   /*  20 SPC */  { CH_USED      ,  (uchar)' '  ,  (uchar)' '  },  /* Space */
   /*  21 !   */  { CH_USED      ,  (uchar)'!'  ,  (uchar)'!'  },  /* Exclamation mark */
   /*  22 "   */  { CH_USED      ,  (uchar)'"'  ,  (uchar)'"'  },  /* Double quote */
   /*  23 #   */  { CH_USED      ,  (uchar)'#'  ,  (uchar)'#'  },  /* Number sign */
   /*  24 CUR */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Currency sign */
   /*  25 %   */  { CH_USED      ,  (uchar)'%'  ,  (uchar)'%'  },  /* Percent sign */
   /*  26 &   */  { CH_USED      ,  (uchar)'&'  ,  (uchar)'&'  },  /* Ampersand */
   /*  27 '   */  { CH_SP_QUOTE  },
   /*  28 (   */  { CH_SP_LPAREN },
   /*  29 )   */  { CH_SP_RPAREN },
   /*  2A *   */  { CH_USED      ,  (uchar)'*'  ,  (uchar)'*'  },  /* Asterisk */
   /*  2B +   */  { CH_SP_PLUS   },
   /*  2C ,   */  { CH_SP_COMMA  },
   /*  2D -   */  { CH_SP_DASH   },
   /*  2E .   */  { CH_SP_PERIOD },
   /*  2F /   */  { CH_SP_SLASH  },
   /*  30 0   */  { CH_DIGIT_0   },
   /*  31 1   */  { CH_DIGIT_1   },
   /*  32 2   */  { CH_DIGIT_2   },
   /*  33 3   */  { CH_DIGIT_3   },
   /*  34 4   */  { CH_DIGIT_4   },
   /*  35 5   */  { CH_DIGIT_5   },
   /*  36 6   */  { CH_DIGIT_6   },
   /*  37 7   */  { CH_DIGIT_7   },
   /*  38 8   */  { CH_DIGIT_8   },
   /*  39 9   */  { CH_DIGIT_9   },
   /*  3A :   */  { CH_SP_COLON  },
   /*  3B ;   */  { CH_USED      ,  (uchar)';'  ,  (uchar)';'  },  /* Semi-colon */
   /*  3C <   */  { CH_USED      ,  (uchar)'<'  ,  (uchar)'<'  },  /* Less than sign */
   /*  3D =   */  { CH_SP_EQUAL  },
   /*  3E >   */  { CH_USED      ,  (uchar)'>'  ,  (uchar)'>'  },  /* Greater than sign */
   /*  3F ?   */  { CH_SP_INTER  },
   /*  40 @   */  { CH_USED      ,  (uchar)'@'  ,  (uchar)'@'  },  /* At sign */
   /*  41 A   */  { CH_UPP_A     },
   /*  42 B   */  { CH_UPP_B     },
   /*  43 C   */  { CH_UPP_C     },
   /*  44 D   */  { CH_UPP_D     },
   /*  45 E   */  { CH_UPP_E     },
   /*  46 F   */  { CH_UPP_F     },
   /*  47 G   */  { CH_UPP_G     },
   /*  48 H   */  { CH_UPP_H     },
   /*  49 I   */  { CH_UPP_I     },
   /*  4A J   */  { CH_UPP_J     },
   /*  4B K   */  { CH_UPP_K     },
   /*  4C L   */  { CH_UPP_L     },
   /*  4D M   */  { CH_UPP_M     },
   /*  4E N   */  { CH_UPP_N     },
   /*  4F O   */  { CH_UPP_O     },
   /*  50 P   */  { CH_UPP_P     },
   /*  51 Q   */  { CH_UPP_Q     },
   /*  52 R   */  { CH_UPP_R     },
   /*  53 S   */  { CH_UPP_S     },
   /*  54 T   */  { CH_UPP_T     },
   /*  55 U   */  { CH_UPP_U     },
   /*  56 V   */  { CH_UPP_V     },
   /*  57 W   */  { CH_UPP_W     },
   /*  58 X   */  { CH_UPP_X     },
   /*  59 Y   */  { CH_UPP_Y     },
   /*  5A Z   */  { CH_UPP_Z     },
   /*  5B [   */  { CH_USED      ,  (uchar)'['  ,  (uchar)'['  },  /* Left bracket */
   /*  5C \   */  { CH_USED      ,  (uchar)'\\' ,  (uchar)'\\' },  /* Back solidus */
   /*  5D ]   */  { CH_USED      ,  (uchar)']'  ,  (uchar)']'  },  /* Right bracket */
   /*  5E ^   */  { CH_USED      ,  (uchar)'^'  ,  (uchar)'^'  },  /* Circumflex accent */
   /*  5F _   */  { CH_USED      ,  (uchar)'_'  ,  (uchar)'_'  },  /* Underscore */
   /*  60 `   */  { CH_USED      ,  (uchar)'`'  ,  (uchar)'`'  },  /* Back quote */
   /*  61 a   */  { CH_LOW_A     },
   /*  62 b   */  { CH_LOW_B     },
   /*  63 c   */  { CH_LOW_C     },
   /*  64 d   */  { CH_LOW_D     },
   /*  65 e   */  { CH_LOW_E     },
   /*  66 f   */  { CH_LOW_F     },
   /*  67 g   */  { CH_LOW_G     },
   /*  68 h   */  { CH_LOW_H     },
   /*  69 i   */  { CH_LOW_I     },
   /*  6A j   */  { CH_LOW_J     },
   /*  6B k   */  { CH_LOW_K     },
   /*  6C l   */  { CH_LOW_L     },
   /*  6D m   */  { CH_LOW_M     },
   /*  6E n   */  { CH_LOW_N     },
   /*  6F o   */  { CH_LOW_O     },
   /*  70 p   */  { CH_LOW_P     },
   /*  71 q   */  { CH_LOW_Q     },
   /*  72 r   */  { CH_LOW_R     },
   /*  73 s   */  { CH_LOW_S     },
   /*  74 t   */  { CH_LOW_T     },
   /*  75 u   */  { CH_LOW_U     },
   /*  76 v   */  { CH_LOW_V     },
   /*  77 w   */  { CH_LOW_W     },
   /*  78 x   */  { CH_LOW_X     },
   /*  79 y   */  { CH_LOW_Y     },
   /*  7A z   */  { CH_LOW_Z     },
   /*  7B {   */  { CH_USED      ,  (uchar)'{'  ,  (uchar)'{'  },  /* Left brace */
   /*  7C |   */  { CH_USED      ,  (uchar)'|'  ,  (uchar)'|'  },  /* Vertical bar */
   /*  7D }   */  { CH_USED      ,  (uchar)'}'  ,  (uchar)'}'  },  /* Right brace */
   /*  7E ~   */  { CH_USED      ,  (uchar)'~'  ,  (uchar)'~'  },  /* Tilda */
#if (0x20 == ' ')
   /*  7F DEL */  { CH_CONTROL   ,  0x7F        ,  0x7F        },  /* ASCII */
#else
   /*  7F DEL */  { CH_CONTROL   ,  0x07        ,  0x07        },  /* EBCDIC */
#endif
   /*  80    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  81    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  82    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  83    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  84    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  85    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  86    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  87    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  88    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  89    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8A    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8B    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8C    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8D    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8E    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  8F    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  90    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  91    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  92    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  93    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  94    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  95    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  96    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  97    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  98    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  99    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9A    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9B    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9C    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9D    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9E    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  9F    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A4    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  A9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AD    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  AF    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
#if (0x20 == ' ')
   /*  B4    */  { CH_UNUSED    ,  0xB4        , 0xB4         },  /* ASCII */
#else
   /*  B4    */  { CH_UNUSED    ,  0x15        , 0x15         },  /* EBCDIC */
#endif
   /*  B5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  B9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BD    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  BF    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C4    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
#if (0x20 == ' ')
   /*  C5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
#else
   /*  C5    */  { CH_UNUSED    ,  0x15        ,  0x15        },  /* EBCDIC */
#endif
   /*  C6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  C9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CD    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  CF    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D4    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  D9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DD    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  DF    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E4    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  E9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  EA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  EB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  EC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  ED    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  EE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  EF    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F0    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F1    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F2    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F3    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F4    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F5    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F6    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F7    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F8    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  F9    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FA    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FB    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FC    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FD    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FE    */  { CH_UNUSED    ,  (uchar)''  ,  (uchar)''  },
   /*  FF     */  { CH_CONTROL   ,  0xFF        ,  0xFF        }   /* same ? */
   };

   /*-------------------------------------------------------------+
   | ISO 8859:1987//CHARSET International Reference Version (IRV) |
   +-------------------------------------------------------------*/
   static CHRDSC ISO_8859_chtab[MAX_CHARSET] = {
   /*  00 N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  01 N/A */  { CH_UNUSED    ,  0x01        ,  0x01        },
   /*  02 N/A */  { CH_UNUSED    ,  0x02        ,  0x02        },
   /*  03 N/A */  { CH_UNUSED    ,  0x03        ,  0x03        },
   /*  04 N/A */  { CH_UNUSED    ,  0x04        ,  0x04        },
   /*  05 N/A */  { CH_UNUSED    ,  0x05        ,  0x05        },
   /*  06 N/A */  { CH_UNUSED    ,  0x06        ,  0x06        },
   /*  07 N/A */  { CH_UNUSED    ,  0x07        ,  0x07        },
   /*  08 N/A */  { CH_UNUSED    ,  0x08        ,  0x08        },
   /*  09 N/A */  { CH_UNUSED    ,  0x09        ,  0x09        },
   /*  0A N/A */  { CH_UNUSED    ,  0x0A        ,  0x0A        },
   /*  0B N/A */  { CH_UNUSED    ,  0x0B        ,  0x0B        },
   /*  0C N/A */  { CH_UNUSED    ,  0x0C        ,  0x0C        },
   /*  0D N/A */  { CH_UNUSED    ,  0x0D        ,  0x0D        },
   /*  0E N/A */  { CH_UNUSED    ,  0x0E        ,  0x0E        },
   /*  0F N/A */  { CH_UNUSED    ,  0x0F        ,  0x0F        },
   /*  10 N/A */  { CH_UNUSED    ,  0x10        ,  0x10        },
   /*  11 N/A */  { CH_UNUSED    ,  0x11        ,  0x11        },
   /*  12 N/A */  { CH_UNUSED    ,  0x12        ,  0x12        },
   /*  13 N/A */  { CH_UNUSED    ,  0x13        ,  0x13        },
   /*  14 N/A */  { CH_UNUSED    ,  0x14        ,  0x14        },
   /*  15 N/A */  { CH_UNUSED    ,  0x15        ,  0x15        },
   /*  16 N/A */  { CH_UNUSED    ,  0x16        ,  0x16        },
   /*  17 N/A */  { CH_UNUSED    ,  0x17        ,  0x17        },
   /*  18 N/A */  { CH_UNUSED    ,  0x18        ,  0x18        },
   /*  19 N/A */  { CH_UNUSED    ,  0x19        ,  0x19        },
   /*  1A N/A */  { CH_UNUSED    ,  0x1A        ,  0x1A        },
   /*  1B N/A */  { CH_UNUSED    ,  0x1B        ,  0x1B        },
   /*  1C N/A */  { CH_UNUSED    ,  0x1C        ,  0x1C        },
   /*  1D N/A */  { CH_UNUSED    ,  0x1D        ,  0x1D        },
   /*  1E N/A */  { CH_UNUSED    ,  0x1E        ,  0x1E        },
   /*  1F N/A */  { CH_UNUSED    ,  0x1F        ,  0x1F        },
   /*  20 SPC */  { CH_USED      ,  (uchar)' '  ,  (uchar)' '  },  /* Space */
   /*  21 !   */  { CH_USED      ,  (uchar)'!'  ,  (uchar)'!'  },  /* Exclamation mark */
   /*  22 "   */  { CH_USED      ,  (uchar)'"'  ,  (uchar)'"'  },  /* Double quote */
   /*  23 #   */  { CH_USED      ,  (uchar)'#'  ,  (uchar)'#'  },  /* Number sign */
   /*  24 $   */  { CH_USED      ,  (uchar)'$'  ,  (uchar)'$'  },  /* Dollar sign */
   /*  25 %   */  { CH_USED      ,  (uchar)'%'  ,  (uchar)'%'  },  /* Percent sign */
   /*  26 &   */  { CH_USED      ,  (uchar)'&'  ,  (uchar)'&'  },  /* Ampersand */
   /*  27 '   */  { CH_SP_QUOTE  },
   /*  28 (   */  { CH_SP_LPAREN },
   /*  29 )   */  { CH_SP_RPAREN },
   /*  2A *   */  { CH_USED      ,  (uchar)'*'  ,  (uchar)'*'  },  /* Asterisk */
   /*  2B +   */  { CH_SP_PLUS   },
   /*  2C ,   */  { CH_SP_COMMA  },
   /*  2D -   */  { CH_SP_DASH   },
   /*  2E .   */  { CH_SP_PERIOD },
   /*  2F /   */  { CH_SP_SLASH  },
   /*  30 0   */  { CH_DIGIT_0   },
   /*  31 1   */  { CH_DIGIT_1   },
   /*  32 2   */  { CH_DIGIT_2   },
   /*  33 3   */  { CH_DIGIT_3   },
   /*  34 4   */  { CH_DIGIT_4   },
   /*  35 5   */  { CH_DIGIT_5   },
   /*  36 6   */  { CH_DIGIT_6   },
   /*  37 7   */  { CH_DIGIT_7   },
   /*  38 8   */  { CH_DIGIT_8   },
   /*  39 9   */  { CH_DIGIT_9   },
   /*  3A :   */  { CH_SP_COLON  },
   /*  3B ;   */  { CH_USED      ,  (uchar)';'  ,  (uchar)';'  },  /* Semi-colon */
   /*  3C <   */  { CH_USED      ,  (uchar)'<'  ,  (uchar)'<'  },  /* Less than sign */
   /*  3D =   */  { CH_SP_EQUAL  },
   /*  3E >   */  { CH_USED      ,  (uchar)'>'  ,  (uchar)'>'  },  /* Greater than sign */
   /*  3F ?   */  { CH_SP_INTER  },
   /*  40 @   */  { CH_USED      ,  (uchar)'@'  ,  (uchar)'@'  },  /* At sign */
   /*  41 A   */  { CH_UPP_A     },
   /*  42 B   */  { CH_UPP_B     },
   /*  43 C   */  { CH_UPP_C     },
   /*  44 D   */  { CH_UPP_D     },
   /*  45 E   */  { CH_UPP_E     },
   /*  46 F   */  { CH_UPP_F     },
   /*  47 G   */  { CH_UPP_G     },
   /*  48 H   */  { CH_UPP_H     },
   /*  49 I   */  { CH_UPP_I     },
   /*  4A J   */  { CH_UPP_J     },
   /*  4B K   */  { CH_UPP_K     },
   /*  4C L   */  { CH_UPP_L     },
   /*  4D M   */  { CH_UPP_M     },
   /*  4E N   */  { CH_UPP_N     },
   /*  4F O   */  { CH_UPP_O     },
   /*  50 P   */  { CH_UPP_P     },
   /*  51 Q   */  { CH_UPP_Q     },
   /*  52 R   */  { CH_UPP_R     },
   /*  53 S   */  { CH_UPP_S     },
   /*  54 T   */  { CH_UPP_T     },
   /*  55 U   */  { CH_UPP_U     },
   /*  56 V   */  { CH_UPP_V     },
   /*  57 W   */  { CH_UPP_W     },
   /*  58 X   */  { CH_UPP_X     },
   /*  59 Y   */  { CH_UPP_Y     },
   /*  5A Z   */  { CH_UPP_Z     },
   /*  5B [   */  { CH_USED      ,  (uchar)'['  ,  (uchar)'['  },  /* Left bracket */
   /*  5C \   */  { CH_USED      ,  (uchar)'\\' ,  (uchar)'\\' },  /* Back solidus */
   /*  5D ]   */  { CH_USED      ,  (uchar)']'  ,  (uchar)']'  },  /* Right bracket */
   /*  5E ^   */  { CH_USED      ,  (uchar)'^'  ,  (uchar)'^'  },  /* Circumflex accent */
   /*  5F _   */  { CH_USED      ,  (uchar)'_'  ,  (uchar)'_'  },  /* Underscore */
   /*  60 `   */  { CH_USED      ,  (uchar)'`'  ,  (uchar)'`'  },  /* Back quote */
   /*  61 a   */  { CH_LOW_A     },
   /*  62 b   */  { CH_LOW_B     },
   /*  63 c   */  { CH_LOW_C     },
   /*  64 d   */  { CH_LOW_D     },
   /*  65 e   */  { CH_LOW_E     },
   /*  66 f   */  { CH_LOW_F     },
   /*  67 g   */  { CH_LOW_G     },
   /*  68 h   */  { CH_LOW_H     },
   /*  69 i   */  { CH_LOW_I     },
   /*  6A j   */  { CH_LOW_J     },
   /*  6B k   */  { CH_LOW_K     },
   /*  6C l   */  { CH_LOW_L     },
   /*  6D m   */  { CH_LOW_M     },
   /*  6E n   */  { CH_LOW_N     },
   /*  6F o   */  { CH_LOW_O     },
   /*  70 p   */  { CH_LOW_P     },
   /*  71 q   */  { CH_LOW_Q     },
   /*  72 r   */  { CH_LOW_R     },
   /*  73 s   */  { CH_LOW_S     },
   /*  74 t   */  { CH_LOW_T     },
   /*  75 u   */  { CH_LOW_U     },
   /*  76 v   */  { CH_LOW_V     },
   /*  77 w   */  { CH_LOW_W     },
   /*  78 x   */  { CH_LOW_X     },
   /*  79 y   */  { CH_LOW_Y     },
   /*  7A z   */  { CH_LOW_Z     },
   /*  7B {   */  { CH_USED      ,  (uchar)'{'  ,  (uchar)'{'  },  /* Left brace */
   /*  7C |   */  { CH_USED      ,  (uchar)'|'  ,  (uchar)'|'  },  /* Vertical bar */
   /*  7D }   */  { CH_USED      ,  (uchar)'}'  ,  (uchar)'}'  },  /* Right brace */
   /*  7E ~   */  { CH_USED      ,  (uchar)'~'  ,  (uchar)'~'  },  /* Tilda */
   /*  7F     */  { CH_UNUSED    ,  0x7F        ,  0x7F        },
   /*  80 N/A */  { CH_UNUSED    ,  0x80        ,  0x80        },
   /*  81 N/A */  { CH_UNUSED    ,  0x81        ,  0x81        },
   /*  82 N/A */  { CH_UNUSED    ,  0x82        ,  0x82        },
   /*  83 N/A */  { CH_UNUSED    ,  0x83        ,  0x83        },
   /*  84 N/A */  { CH_UNUSED    ,  0x84        ,  0x84        },
   /*  85 N/A */  { CH_UNUSED    ,  0x85        ,  0x85        },
   /*  86 N/A */  { CH_UNUSED    ,  0x86        ,  0x86        },
   /*  87 N/A */  { CH_UNUSED    ,  0x87        ,  0x87        },
   /*  88 N/A */  { CH_UNUSED    ,  0x88        ,  0x88        },
   /*  89 N/A */  { CH_UNUSED    ,  0x89        ,  0x89        },
   /*  8A N/A */  { CH_UNUSED    ,  0x8A        ,  0x8A        },
   /*  8B N/A */  { CH_UNUSED    ,  0x8B        ,  0x8B        },
   /*  8C N/A */  { CH_UNUSED    ,  0x8C        ,  0x8C        },
   /*  8D N/A */  { CH_UNUSED    ,  0x8D        ,  0x8D        },
   /*  8E N/A */  { CH_UNUSED    ,  0x8E        ,  0x8E        },
   /*  8F N/A */  { CH_UNUSED    ,  0x8F        ,  0x8F        },
   /*  90 N/A */  { CH_UNUSED    ,  0x90        ,  0x90        },
   /*  91 N/A */  { CH_UNUSED    ,  0x91        ,  0x91        },
   /*  92 N/A */  { CH_UNUSED    ,  0x92        ,  0x92        },
   /*  93 N/A */  { CH_UNUSED    ,  0x93        ,  0x93        },
   /*  94 N/A */  { CH_UNUSED    ,  0x94        ,  0x94        },
   /*  95 N/A */  { CH_UNUSED    ,  0x95        ,  0x95        },
   /*  96 N/A */  { CH_UNUSED    ,  0x96        ,  0x96        },
   /*  97 N/A */  { CH_UNUSED    ,  0x97        ,  0x97        },
   /*  98 N/A */  { CH_UNUSED    ,  0x98        ,  0x98        },
   /*  99 N/A */  { CH_UNUSED    ,  0x99        ,  0x99        },
   /*  9A N/A */  { CH_UNUSED    ,  0x9A        ,  0x9A        },
   /*  9B N/A */  { CH_UNUSED    ,  0x9B        ,  0x9B        },
   /*  9C N/A */  { CH_UNUSED    ,  0x9C        ,  0x9C        },
   /*  9D N/A */  { CH_UNUSED    ,  0x9D        ,  0x9D        },
   /*  9E N/A */  { CH_UNUSED    ,  0x9E        ,  0x9E        },
   /*  9F N/A */  { CH_UNUSED    ,  0x9F        ,  0x9F        },
   /*  A0    */  { CH_USED      ,  0xFF        ,  0xFF        },  /* NBSP: Required blank ??? */
   /*  A1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted exclamation mark */
   /*  A2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cent sign */
   /*  A3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pound sign */
   /*  A4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Currency sign */
   /*  A5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Yen sign */
   /*  A6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Broken bar */
   /*  A7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Paragraph Sign, section Sign */
   /*  A8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Diaeresis */
   /*  A9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Copyright sign */
   /*  AA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Feminine ordinal indicator */
   /*  AB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left angle quotation mark */
   /*  AC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Not sign */
   /*  AD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* SHY: Soft hyphen ??? */
   /*  AE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Registered Trade Mark sign */
   /*  AF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Macron */
   /*  B0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Ring above, degree sign */
   /*  B1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Plus-minus sign
   /*  B2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 2 */
   /*  B3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 3 */
   /*  B4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Acute accent */
   /*  B5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Micro sign */
   /*  B6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pilcrow sign */
   /*  B7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Middle dot */
   /*  B8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cedilla */
   /*  B9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 1 */
   /*  BA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Masculine ordinal indicator */
   /*  BB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Right angle quotation mark */
   /*  BC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one quarter */
   /*  BD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one half */
   /*  BE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction three quarters */
   /*  BF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted question mark */
   /*  C0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with grave accent */
   /*  C1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with acute accent */
   /*  C2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with circumflex accent */
   /*  C3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with tilde */
   /*  C4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with diaeresis */
   /*  C5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with ring above */
   /*  C6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital diphtong A with E */
   /*  C7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter C with cedilla */
   /*  C8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with grave accent */
   /*  C9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with acute accent */
   /*  CA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with circumflex accent */
   /*  CB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with diaeresis */
   /*  CC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with grave accent */
   /*  CD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with acute accent */
   /*  CE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with circumflex accent */
   /*  CF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with diaeresis */
   /*  D0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital icelandic letter ETH */
   /*  D1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter N with tilde */
   /*  D2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with grave accent */
   /*  D3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with acute accent */
   /*  D4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with circumflex accent */
   /*  D5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with tilde */
   /*  D6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with diaeresis */
   /*  D7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Multiplication sign */
   /*  D8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with oblique stroke */
   /*  D9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with grave accent */
   /*  DA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with acute accent */
   /*  DB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with circumflex accent */
   /*  DC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with diaeresis */
   /*  DD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter Y with acute accent */
   /*  DE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital icelandic letter THORN */
   /*  DF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small german letter sharp S */
   /*  E0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with grave accent */
   /*  E1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with acute accent */
   /*  E2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with circumflex accent */
   /*  E3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with tilde */
   /*  E4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with diaeresis */
   /*  E5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with ring above */
   /*  E6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small diphtong A with E */
   /*  E7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter C with cedilla */
   /*  E8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with grave accent */
   /*  E9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with acute accent */
   /*  EA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with circumflex accent *
   /*  EB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with diaeresis */
   /*  EC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with grave accent */
   /*  ED    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with acute accent */
   /*  EE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with circumflex accent *
   /*  EF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with diaeresis */
   /*  F0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small icelandic letter ETH */
   /*  F1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter N with tilde */
   /*  F2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with grave accent */
   /*  F3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with acute accent */
   /*  F4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with circumflex accent *
   /*  F5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with tilde */
   /*  F6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with diaeresis */
   /*  F7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Division sign */
   /*  F8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with oblique stroke */
   /*  F9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with grave accent */
   /*  FA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with acute accent */
   /*  FB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with circumflex accent *
   /*  FC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with diaeresis */
   /*  FD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter Y with acute accent */
   /*  FE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small icelandic letter THORN */
   /*  FF     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  }   /* Small letter Y with diaeresis */
   };

   /*-------------------------------------------------------------+
   | IBM 395                                                      |
   +-------------------------------------------------------------*/
   static CHRDSC IBM_395_chtab[MAX_CHARSET] = {
   /*  00 N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  01 N/A */  { CH_UNUSED    ,  0x01        ,  0x01        },
   /*  02 N/A */  { CH_UNUSED    ,  0x02        ,  0x02        },
   /*  03 N/A */  { CH_UNUSED    ,  0x03        ,  0x03        },
   /*  04 N/A */  { CH_UNUSED    ,  0x04        ,  0x04        },
   /*  05 N/A */  { CH_UNUSED    ,  0x05        ,  0x05        },
   /*  06 N/A */  { CH_UNUSED    ,  0x06        ,  0x06        },
   /*  07 N/A */  { CH_UNUSED    ,  0x07        ,  0x07        },
   /*  08 N/A */  { CH_UNUSED    ,  0x08        ,  0x08        },
   /*  09 N/A */  { CH_UNUSED    ,  0x09        ,  0x09        },
   /*  0A N/A */  { CH_UNUSED    ,  0x0A        ,  0x0A        },
   /*  0B N/A */  { CH_UNUSED    ,  0x0B        ,  0x0B        },
   /*  0C N/A */  { CH_UNUSED    ,  0x0C        ,  0x0C        },
   /*  0D N/A */  { CH_UNUSED    ,  0x0D        ,  0x0D        },
   /*  0E N/A */  { CH_UNUSED    ,  0x0E        ,  0x0E        },
   /*  0F N/A */  { CH_UNUSED    ,  0x0F        ,  0x0F        },
   /*  10 N/A */  { CH_UNUSED    ,  0x10        ,  0x10        },
   /*  11 N/A */  { CH_UNUSED    ,  0x11        ,  0x11        },
   /*  12 N/A */  { CH_UNUSED    ,  0x12        ,  0x12        },
   /*  13 N/A */  { CH_UNUSED    ,  0x13        ,  0x13        },
   /*  14 N/A */  { CH_UNUSED    ,  0x14        ,  0x14        },
   /*  15 N/A */  { CH_UNUSED    ,  0x15        ,  0x15        },
   /*  16 N/A */  { CH_UNUSED    ,  0x16        ,  0x16        },
   /*  17 N/A */  { CH_UNUSED    ,  0x17        ,  0x17        },
   /*  18 N/A */  { CH_UNUSED    ,  0x18        ,  0x18        },
   /*  19 N/A */  { CH_UNUSED    ,  0x19        ,  0x19        },
   /*  1A N/A */  { CH_UNUSED    ,  0x1A        ,  0x1A        },
   /*  1B N/A */  { CH_UNUSED    ,  0x1B        ,  0x1B        },
   /*  1C N/A */  { CH_UNUSED    ,  0x1C        ,  0x1C        },
   /*  1D N/A */  { CH_UNUSED    ,  0x1D        ,  0x1D        },
   /*  1E N/A */  { CH_UNUSED    ,  0x1E        ,  0x1E        },
   /*  1F N/A */  { CH_UNUSED    ,  0x1F        ,  0x1F        },
   /*  20 N/A */  { CH_UNUSED    ,  0x20        ,  0x20        },
   /*  21 N/A */  { CH_UNUSED    ,  0x21        ,  0x21        },
   /*  22 N/A */  { CH_UNUSED    ,  0x22        ,  0x22        },
   /*  23 N/A */  { CH_UNUSED    ,  0x23        ,  0x23        },
   /*  24 N/A */  { CH_UNUSED    ,  0x24        ,  0x24        },
   /*  25 N/A */  { CH_UNUSED    ,  0x25        ,  0x25        },
   /*  26 N/A */  { CH_UNUSED    ,  0x26        ,  0x26        },
   /*  27 N/A */  { CH_UNUSED    ,  0x27        ,  0x27        },
   /*  28 N/A */  { CH_UNUSED    ,  0x28        ,  0x28        },
   /*  29 N/A */  { CH_UNUSED    ,  0x29        ,  0x29        },
   /*  2A N/A */  { CH_UNUSED    ,  0x2A        ,  0x2A        },
   /*  2B N/A */  { CH_UNUSED    ,  0x2B        ,  0x2B        },
   /*  2C N/A */  { CH_UNUSED    ,  0x2C        ,  0x2C        },
   /*  2D N/A */  { CH_UNUSED    ,  0x2D        ,  0x2D        },
   /*  2E N/A */  { CH_UNUSED    ,  0x2E        ,  0x2E        },
   /*  2F N/A */  { CH_UNUSED    ,  0x2F        ,  0x2F        },
   /*  30 N/A */  { CH_UNUSED    ,  0x30        ,  0x30        },
   /*  31 N/A */  { CH_UNUSED    ,  0x31        ,  0x31        },
   /*  32 N/A */  { CH_UNUSED    ,  0x32        ,  0x32        },
   /*  33 N/A */  { CH_UNUSED    ,  0x33        ,  0x33        },
   /*  34 N/A */  { CH_UNUSED    ,  0x34        ,  0x34        },
   /*  35 N/A */  { CH_UNUSED    ,  0x35        ,  0x35        },
   /*  36 N/A */  { CH_UNUSED    ,  0x36        ,  0x36        },
   /*  37 N/A */  { CH_UNUSED    ,  0x37        ,  0x37        },
   /*  38 N/A */  { CH_UNUSED    ,  0x38        ,  0x38        },
   /*  39 N/A */  { CH_UNUSED    ,  0x39        ,  0x39        },
   /*  3A N/A */  { CH_UNUSED    ,  0x3A        ,  0x3A        },
   /*  3B N/A */  { CH_UNUSED    ,  0x3B        ,  0x3B        },
   /*  3C N/A */  { CH_UNUSED    ,  0x3C        ,  0x3C        },
   /*  3D N/A */  { CH_UNUSED    ,  0x3D        ,  0x3D        },
   /*  3E N/A */  { CH_UNUSED    ,  0x3E        ,  0x3E        },
   /*  3F N/A */  { CH_UNUSED    ,  0x3F        ,  0x3F        },
   /*  40 SPC */  { CH_USED      ,  (uchar)' '  ,  (uchar)' '  },  /* Space */
#if (0x20 == ' ')                                                  /* NBSP: Required blank ??? */
   /*  41     */  { CH_USED      ,  0xFF        ,  0xFF        },  /* ASCII */
#else
   /*  41     */  { CH_USED      ,  0x41        ,  0x41        },  /* EBCDIC */
#endif
   /*  42     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with circumflex accent */
   /*  43     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with diaeresis */
   /*  44     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with grave accent */
   /*  45     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with acute accent */
   /*  46     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with tilde */
   /*  47     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with ring above */
   /*  48     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter C with cedilla */
   /*  49     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter N with tilde */
   /*  4A     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cent sign */
   /*  4B     */  { CH_SP_PERIOD },
   /*  4C     */  { CH_USED      ,  (uchar)'<'  ,  (uchar)'<'  },  /* Less than sign */
   /*  4D     */  { CH_SP_LPAREN },
   /*  4E     */  { CH_SP_PLUS   },
   /*  4F     */  { CH_USED      ,  (uchar)'|'  ,  (uchar)'|'  },  /* Vertical bar */
   /*  50     */  { CH_USED      ,  (uchar)'&'  ,  (uchar)'&'  },  /* Ampersand */
   /*  51     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with acute accent */
   /*  52     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with circumflex accent *
   /*  53     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with diaeresis */
   /*  54     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with grave accent */
   /*  55     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with acute accent */
   /*  56     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with circumflex accent *
   /*  57     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with diaeresis */
   /*  58     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with grave accent */
   /*  59     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small German Letter Sharp s */
   /*  5A     */  { CH_USED      ,  (uchar)'!'  ,  (uchar)'!'  },  /* Exclamation mark */
   /*  5B     */  { CH_USED      ,  (uchar)'$'  ,  (uchar)'$'  },  /* Dollar sign */
   /*  5C     */  { CH_USED      ,  (uchar)'*'  ,  (uchar)'*'  },  /* Asterisk */
   /*  5D     */  { CH_SP_RPAREN },
   /*  5E     */  { CH_USED      ,  (uchar)';'  ,  (uchar)';'  },  /* Semi-colon */
   /*  5F N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  60    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* SHY: Soft hyphen ??? */
   /*  61     */  { CH_SP_SLASH  },
   /*  62     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with circumflex accent *
   /*  63     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with diaeresis */
   /*  64     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with grave accent */
   /*  65     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with acute accent */
   /*  66     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with tilde */
   /*  67     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with ring above */
   /*  68     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter C with cedilla */
   /*  69     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter N with tilde */
   /*  6A     */  { CH_USED      ,  (uchar)']'  ,  (uchar)']'  },  /* Right bracket */
   /*  6B     */  { CH_SP_COMMA  },
   /*  6C     */  { CH_USED      ,  (uchar)'%'  ,  (uchar)'%'  },  /* Percent sign */
   /*  6D N/A */  { CH_UNUSED    ,  0x00        , 0x00         },
   /*  6E     */  { CH_USED      ,  (uchar)'>'  ,  (uchar)'>'  },  /* Greater than sign */
   /*  6F     */  { CH_SP_INTER  },
   /*  70     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with oblique stroke */
   /*  71     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with acute accent */
   /*  72     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with circumflex accent */
   /*  73     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with diaeresis */
   /*  74     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with grave accent */
   /*  75     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with acute accent */
   /*  76     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with circumflex accent */
   /*  77     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with diaeresis */
   /*  78     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with grave accent */
   /*  79 N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  7A     */  { CH_SP_COLON  },
   /*  7B     */  { CH_USED      ,  (uchar)'#'  ,  (uchar)'#'  },  /* Number sign */
   /*  7C     */  { CH_USED      ,  (uchar)'@'  ,  (uchar)'@'  },  /* At sign */
   /*  7D     */  { CH_SP_QUOTE  },
   /*  7E =   */  { CH_SP_EQUAL  },
   /*  7F     */  { CH_USED      ,  (uchar)'"'  ,  (uchar)'"'  },  /* Double quote */
   /*  80     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with oblique stroke */
   /*  81 a   */  { CH_LOW_A     },
   /*  82 b   */  { CH_LOW_B     },
   /*  83 c   */  { CH_LOW_C     },
   /*  84 d   */  { CH_LOW_D     },
   /*  85 e   */  { CH_LOW_E     },
   /*  86 f   */  { CH_LOW_F     },
   /*  87 g   */  { CH_LOW_G     },
   /*  88 h   */  { CH_LOW_H     },
   /*  89 i   */  { CH_LOW_I     },
   /*  8A     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left angle quotation mark */
   /*  8B     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Right angle quotation mark */
   /*  8C     */                                                   /* Per mill sign */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0x8C        ,  0x8C        },  /* EBCDIC */
#endif
   /*  8D     */  { CH_SP_DASH   },
   /*  8E     */                                                   /* ij ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0x8E        ,  0x8E        },  /* EBCDIC */
#endif
   /*  8F     */                                                   /* ffi ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0x8F        ,  0x8F        },  /* EBCDIC */
#endif
   /*  90     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Ring above, degree sign */
   /*  91 j   */  { CH_LOW_J     },
   /*  92 k   */  { CH_LOW_K     },
   /*  93 l   */  { CH_LOW_L     },
   /*  94 m   */  { CH_LOW_M     },
   /*  95 n   */  { CH_LOW_N     },
   /*  96 o   */  { CH_LOW_O     },
   /*  97 p   */  { CH_LOW_P     },
   /*  98 q   */  { CH_LOW_Q     },
   /*  99 r   */  { CH_LOW_R     },
   /*  9A     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Macron */
   /*  9B     */                                                   /* Capital diphtong O with E */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0x9B        ,  0x9B        },  /* EBCDIC */
#endif
   /*  9C     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small diphtong A with E */
   /*  9D     */                                                   /* Capital letter Y with diaeresis */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0x9D        ,  0x9D        },  /* EBCDIC */
#endif
   /*  9E     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital diphtong A with E */
   /*  9F     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Middle dot */
   /*  A0     */                                                   /* ff ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xA0        ,  0xA0        },  /* EBCDIC */
#endif
   /*  A1 N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  A2 s   */  { CH_LOW_S     },
   /*  A3 t   */  { CH_LOW_T     },
   /*  A4 u   */  { CH_LOW_U     },
   /*  A5 v   */  { CH_LOW_V     },
   /*  A6 w   */  { CH_LOW_W     },
   /*  A7 x   */  { CH_LOW_X     },
   /*  A8 y   */  { CH_LOW_Y     },
   /*  A9 z   */  { CH_LOW_Z     },
   /*  AA     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted exclamation mark */
   /*  AB     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted question mark */
   /*  AC     */                                                   /* Small diphtong O with E */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xAC        ,  0xAC        },  /* EBCDIC */
#endif
   /*  AD     */                                                   /* ffl ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xAD        ,  0xAD        },  /* EBCDIC */
#endif
   /*  AE     */                                                   /* Plus over plus */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xAE        ,  0xAE        },  /* EBCDIC */
#endif
   /*  AF     */                                                   /* Low right double quotation mark */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xAF        ,  0xAF        },  /* EBCDIC */
#endif
   /*  B0     */  { CH_USED      ,  (uchar)'['  ,  (uchar)'['  },  /* Left bracket */
   /*  B1     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pound sign */
   /*  B2     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Yen sign */
   /*  B3     */                                                   /* fi ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xB3        ,  0xB3        },  /* EBCDIC */
#endif
   /*  B4     */                                                   /* fl ligature */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xB4        ,  0xB4        },  /* EBCDIC */
#endif
   /*  B5     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Paragraph Sign, section Sign */
   /*  B6     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pilcrow sign */
   /*  B7     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one quarter */
   /*  B8     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one half */
   /*  B9     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction three quarters */
   /*  BA     */                                                   /* Vulgar fraction one eighth */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBA        ,  0xBA        },  /* EBCDIC */
#endif
   /*  BB     */                                                   /* Left single quotation mark */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBB        ,  0xBB        },  /* EBCDIC */
#endif
   /*  BC     */                                                   /* Right single quotation mark */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBC        ,  0xBC        },  /* EBCDIC */
#endif
   /*  BD     */                                                   /* Left double quotation mark */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBD        ,  0xBD        },  /* EBCDIC */
#endif
   /*  BE     */                                                   /* Right double quotation mark */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBE        ,  0xBE        },  /* EBCDIC */
#endif
   /*  BF     */                                                   /* Dagger */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xBF        ,  0xBF        },  /* EBCDIC */
#endif
   /*  C0     */  { CH_USED      ,  (uchar)'{'  ,  (uchar)'{'  },  /* Left brace */
   /*  C1 A   */  { CH_UPP_A     },
   /*  C2 B   */  { CH_UPP_B     },
   /*  C3 C   */  { CH_UPP_C     },
   /*  C4 D   */  { CH_UPP_D     },
   /*  C5 E   */  { CH_UPP_E     },
   /*  C6 F   */  { CH_UPP_F     },
   /*  C7 G   */  { CH_UPP_G     },
   /*  C8 H   */  { CH_UPP_H     },
   /*  C9 I   */  { CH_UPP_I     },
   /*  CA N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  CB     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with circumflex accent */
   /*  CC     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with diaeresis */
   /*  CD     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with grave accent */
   /*  CE     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with acute accent */
   /*  CF     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with tilde */
   /*  D0     */  { CH_USED      ,  (uchar)'}'  ,  (uchar)'}'  },  /* Right brace */
   /*  D1 J   */  { CH_UPP_J     },
   /*  D2 K   */  { CH_UPP_K     },
   /*  D3 L   */  { CH_UPP_L     },
   /*  D4 M   */  { CH_UPP_M     },
   /*  D5 N   */  { CH_UPP_N     },
   /*  D6 O   */  { CH_UPP_O     },
   /*  D7 P   */  { CH_UPP_P     },
   /*  D8 Q   */  { CH_UPP_Q     },
   /*  D9 R   */  { CH_UPP_R     },
   /*  DA     */                                                   /* Vulgar fraction three eighth */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xDA        ,  0xDA        },  /* EBCDIC */
#endif
   /*  DB     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with circumflex accent */
   /*  DC     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with diaeresis */
   /*  DD     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with grave accent */
   /*  DE     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with acute accent */
   /*  DF     */  { CH_CONTROL   ,  (uchar)''  ,  (uchar)''  },  /* Small letter Y with diaeresis */
   /*  E0     */  { CH_USED      ,  (uchar)'\\' ,  (uchar)'\\' },  /* Back solidus */
   /*  E1 N/A */  { CH_UNUSED    ,  0x00        ,  0x00        },
   /*  E2 S   */  { CH_UPP_S     },
   /*  E3 T   */  { CH_UPP_T     },
   /*  E4 U   */  { CH_UPP_U     },
   /*  E5 V   */  { CH_UPP_V     },
   /*  E6 W   */  { CH_UPP_W     },
   /*  E7 X   */  { CH_UPP_X     },
   /*  E8 Y   */  { CH_UPP_Y     },
   /*  E9 Z   */  { CH_UPP_Z     },
   /*  EA     */                                                   /* Vulgar fraction five eighth */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xEA        ,  0xEA        },  /* EBCDIC */
#endif
   /*  EB     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with circumflex accent */
   /*  EC     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with diaeresis */
   /*  ED     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with grave accent */
   /*  EE     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with acute accent */
   /*  EF     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with tilde */
   /*  F0 0   */  { CH_DIGIT_0   },
   /*  F1 1   */  { CH_DIGIT_1   },
   /*  F2 2   */  { CH_DIGIT_2   },
   /*  F3 3   */  { CH_DIGIT_3   },
   /*  F4 4   */  { CH_DIGIT_4   },
   /*  F5 5   */  { CH_DIGIT_5   },
   /*  F6 6   */  { CH_DIGIT_6   },
   /*  F7 7   */  { CH_DIGIT_7   },
   /*  F8 8   */  { CH_DIGIT_8   },
   /*  F9 9   */  { CH_DIGIT_9   },
   /*  FA     */                                                   /* Vulgar fraction five eighth */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xFA        ,  0xFA        },  /* EBCDIC */
#endif
   /*  FB     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with circumflex accent */
   /*  FC     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with diaeresis */
   /*  FD     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with grave accent */
   /*  FE     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with acute accent */
   /*  FF     */                                                   /* Capital diphtong E with O */
#if (0x20 == ' ')
                  { CH_UNUSED    ,  0x00        ,  0x00        },  /* ASCII 850: doesn't exist */
#else
                  { CH_USED      ,  0xFF        ,  0xFF        },  /* EBCDIC */
#endif
   };

   /*-------------------------------------------------------------+
   | IBM 850                                                      |
   +-------------------------------------------------------------*/
   static CHRDSC IBM_850_chtab[MAX_CHARSET] = {
   /*  00 NUL */  { CH_CONTROL   ,  0x00        ,  0x00        },  /* CONTROL */
   /*  01    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* White happy face */
   /*  02    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Black happy face */
   /*  03    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Heart */
   /*  04    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Diamond */
   /*  05    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Club */
   /*  06    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Spade */
   /*  07    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Big dot */
   /*  08    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Reverse big dot */
   /*  09     */  { CH_USED      ,  0x09        ,  0x09        },  /* Circle */
   /*  0A     */  { CH_USED      ,  0x0A        ,  0x0A        },  /* Reverse circle */
   /*  0B    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Masculine symbol */
   /*  0C    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Feminine symbol */
   /*  0D     */  { CH_USED      ,  0x0D        ,  0x0D        },  /* Single music note */
   /*  0E    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Two music notes */
   /*  0F    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Sun */
   /*  10    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Right triangle */
   /*  11    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left triangle */
   /*  12    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Double vertical arrow */
   /*  13    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Double exclamation mark */
   /*  14    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pilcrow sign */
   /*  15    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Paragraph sign */
   /*  16    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Fat hyphen */
   /*  17    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Double vertical arrow underlined */
   /*  18    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Up arrow */
   /*  19    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Down arrow */
   /*  1A     */  { CH_CONTROL   ,  0x1A        ,  0x1A        },  /* Right arrow */
   /*  1B    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left arrow */
   /*  1C    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted not sign */
   /*  1D    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Double horizontal arrow */
   /*  1E    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Up triangle */
   /*  1F    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Down triangle */
   /*  20 SPC */  { CH_USED      ,  (uchar)' '  ,  (uchar)' '  },  /* Space */
   /*  21 !   */  { CH_USED      ,  (uchar)'!'  ,  (uchar)'!'  },  /* Exclamation mark */
   /*  22 "   */  { CH_USED      ,  (uchar)'"'  ,  (uchar)'"'  },  /* Double quote */
   /*  23 #   */  { CH_USED      ,  (uchar)'#'  ,  (uchar)'#'  },  /* Number sign */
   /*  24 $   */  { CH_USED      ,  (uchar)'$'  ,  (uchar)'$'  },  /* Dollar sign */
   /*  25 %   */  { CH_USED      ,  (uchar)'%'  ,  (uchar)'%'  },  /* Percent sign */
   /*  26 &   */  { CH_USED      ,  (uchar)'&'  ,  (uchar)'&'  },  /* Ampersand */
   /*  27 '   */  { CH_SP_QUOTE  },
   /*  28 (   */  { CH_SP_LPAREN },
   /*  29 )   */  { CH_SP_RPAREN },
   /*  2A *   */  { CH_USED      ,  (uchar)'*'  ,  (uchar)'*'  },  /* Asterisk */
   /*  2B +   */  { CH_SP_PLUS   },
   /*  2C ,   */  { CH_SP_COMMA  },
   /*  2D -   */  { CH_SP_DASH   },
   /*  2E .   */  { CH_SP_PERIOD },
   /*  2F /   */  { CH_SP_SLASH  },
   /*  30 0   */  { CH_DIGIT_0   },
   /*  31 1   */  { CH_DIGIT_1   },
   /*  32 2   */  { CH_DIGIT_2   },
   /*  33 3   */  { CH_DIGIT_3   },
   /*  34 4   */  { CH_DIGIT_4   },
   /*  35 5   */  { CH_DIGIT_5   },
   /*  36 6   */  { CH_DIGIT_6   },
   /*  37 7   */  { CH_DIGIT_7   },
   /*  38 8   */  { CH_DIGIT_8   },
   /*  39 9   */  { CH_DIGIT_9   },
   /*  3A :   */  { CH_SP_COLON  },
   /*  3B ;   */  { CH_USED      ,  (uchar)';'  ,  (uchar)';'  },  /* Semi-colon */
   /*  3C <   */  { CH_USED      ,  (uchar)'<'  ,  (uchar)'<'  },  /* Less than sign */
   /*  3D =   */  { CH_SP_EQUAL  },
   /*  3E >   */  { CH_USED      ,  (uchar)'>'  ,  (uchar)'>'  },  /* Greater than sign */
   /*  3F ?   */  { CH_SP_INTER  },
   /*  40 @   */  { CH_USED      ,  (uchar)'@'  ,  (uchar)'@'  },  /* At sign */
   /*  41 A   */  { CH_UPP_A     },
   /*  42 B   */  { CH_UPP_B     },
   /*  43 C   */  { CH_UPP_C     },
   /*  44 D   */  { CH_UPP_D     },
   /*  45 E   */  { CH_UPP_E     },
   /*  46 F   */  { CH_UPP_F     },
   /*  47 G   */  { CH_UPP_G     },
   /*  48 H   */  { CH_UPP_H     },
   /*  49 I   */  { CH_UPP_I     },
   /*  4A J   */  { CH_UPP_J     },
   /*  4B K   */  { CH_UPP_K     },
   /*  4C L   */  { CH_UPP_L     },
   /*  4D M   */  { CH_UPP_M     },
   /*  4E N   */  { CH_UPP_N     },
   /*  4F O   */  { CH_UPP_O     },
   /*  50 P   */  { CH_UPP_P     },
   /*  51 Q   */  { CH_UPP_Q     },
   /*  52 R   */  { CH_UPP_R     },
   /*  53 S   */  { CH_UPP_S     },
   /*  54 T   */  { CH_UPP_T     },
   /*  55 U   */  { CH_UPP_U     },
   /*  56 V   */  { CH_UPP_V     },
   /*  57 W   */  { CH_UPP_W     },
   /*  58 X   */  { CH_UPP_X     },
   /*  59 Y   */  { CH_UPP_Y     },
   /*  5A Z   */  { CH_UPP_Z     },
   /*  5B [   */  { CH_USED      ,  (uchar)'['  ,  (uchar)'['  },  /* Left bracket */
   /*  5C \   */  { CH_USED      ,  (uchar)'\\' ,  (uchar)'\\' },  /* Back solidus */
   /*  5D ]   */  { CH_USED      ,  (uchar)']'  ,  (uchar)']'  },  /* Right bracket */
   /*  5E ^   */  { CH_USED      ,  (uchar)'^'  ,  (uchar)'^'  },  /* Circumflex accent */
   /*  5F _   */  { CH_USED      ,  (uchar)'_'  ,  (uchar)'_'  },  /* Underscore */
   /*  60 `   */  { CH_USED      ,  (uchar)'`'  ,  (uchar)'`'  },  /* Back quote */
   /*  61 a   */  { CH_LOW_A     },
   /*  62 b   */  { CH_LOW_B     },
   /*  63 c   */  { CH_LOW_C     },
   /*  64 d   */  { CH_LOW_D     },
   /*  65 e   */  { CH_LOW_E     },
   /*  66 f   */  { CH_LOW_F     },
   /*  67 g   */  { CH_LOW_G     },
   /*  68 h   */  { CH_LOW_H     },
   /*  69 i   */  { CH_LOW_I     },
   /*  6A j   */  { CH_LOW_J     },
   /*  6B k   */  { CH_LOW_K     },
   /*  6C l   */  { CH_LOW_L     },
   /*  6D m   */  { CH_LOW_M     },
   /*  6E n   */  { CH_LOW_N     },
   /*  6F o   */  { CH_LOW_O     },
   /*  70 p   */  { CH_LOW_P     },
   /*  71 q   */  { CH_LOW_Q     },
   /*  72 r   */  { CH_LOW_R     },
   /*  73 s   */  { CH_LOW_S     },
   /*  74 t   */  { CH_LOW_T     },
   /*  75 u   */  { CH_LOW_U     },
   /*  76 v   */  { CH_LOW_V     },
   /*  77 w   */  { CH_LOW_W     },
   /*  78 x   */  { CH_LOW_X     },
   /*  79 y   */  { CH_LOW_Y     },
   /*  7A z   */  { CH_LOW_Z     },
   /*  7B {   */  { CH_USED      ,  (uchar)'{'  ,  (uchar)'{'  },  /* Left brace */
   /*  7C |   */  { CH_USED      ,  (uchar)'|'  ,  (uchar)'|'  },  /* Vertical bar */
   /*  7D }   */  { CH_USED      ,  (uchar)'}'  ,  (uchar)'}'  },  /* Right brace */
   /*  7E ~   */  { CH_USED      ,  (uchar)'~'  ,  (uchar)'~'  },  /* Tilda */
   /*  7F    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* House */
   /*  80    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter C with Cedilla */
   /*  81    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter u with Diaeresis */
   /*  82    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter e with Acute Accent */
   /*  83    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Circumflex Accent */
   /*  84    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Diaeresis */
   /*  85    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Grave Accent */
   /*  86    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Ring Above */
   /*  87    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter c with Cedilla */
   /*  88    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter e with Circumflex Accent */
   /*  89    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter e with Diaeresis */
   /*  8A    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter e with Grave Accent */
   /*  8B    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter i with Diaeresis */
   /*  8C    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter i with Circumflex */
   /*  8D    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter i with Grave Accent */
   /*  8E    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Diaeresis */
   /*  8F    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Ring Above */
   /*  90    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter E with Acute Accent */
   /*  91    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Dipthong a with e */
   /*  92    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Dipthong A with E */
   /*  93    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Circumflex Accent */
   /*  94    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Diaeresis */
   /*  95    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Grave Accent */
   /*  96    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter u with Circumflex Accent */
   /*  97    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter u with Grave Accent */
   /*  98    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter y with Diaeresis */
   /*  99    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Diaeresis */
   /*  9A    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter U with Diaeresis */
   /*  9B    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Oblique Stroke */
   /*  9C    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pound Sign */
   /*  9D    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Oblique Stroke */
   /*  9E    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Multiplication Sign */
   /*  9F    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Function Of */
   /*  A0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Acute Accent */
   /*  A1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter i with Acute Accent */
   /*  A2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Acute Accent */
   /*  A3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter u with Acute Accent */
   /*  A4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter n with Tilde */
   /*  A5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter N with Tilde */
   /*  A6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Feminine Ordinal Indicator */
   /*  A7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Masculine Ordinal Indicator */
   /*  A8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted Question Mark */
   /*  A9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Registered Trade Mark Sign */
   /*  AA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Not Sign */
   /*  AB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar Fraction One Half */
   /*  AC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar Fraction One Quarter */
   /*  AD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted Exclamation Mark */
   /*  AE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left Angle Quotation Mark */
   /*  AF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Right Angle Quotation Mark */
   /*  B0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* 1/4 shaded block */
   /*  B1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* 1/2 shaded block */
   /*  B2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* 3/4 shaded block */
   /*  B3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxv - Vertical line */
   /*  B4    */  { CH_USED      ,  0xB4        ,  0xB4        },  /* boxvl */
   /*  B5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Acute Accent */
   /*  B6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Circumflex Accent */
   /*  B7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Grave Accent */
   /*  B8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Copyright Sign */
   /*  B9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxVL */
   /*  BA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxV */
   /*  BB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxDL */
   /*  BC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxUL */
   /*  BD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cent Sign */
   /*  BE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Yen Sign */
   /*  BF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxdl */
   /*  C0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxur */
   /*  C1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxuh */
   /*  C2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxdh */
   /*  C3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxvr */
   /*  C4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxh */
   /*  C5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxvh */
   /*  C6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter a with Tilde */
   /*  C7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter A with Tilde */
   /*  C8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxUR */
   /*  C9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxDR */
   /*  CA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxUH */
   /*  CB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxDH */
   /*  CC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxVR */
   /*  CD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxH */
   /*  CE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxVH */
   /*  CF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Currency Sign */
   /*  D0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Icelandic Letter Thorn */
   /*  D1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Icelandic Letter Thorn */
   /*  D2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter E with Circumflex Accent */
   /*  D3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter E with Diaeresis */
   /*  D4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter E with Grave Accent */
   /*  D5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter l reduced (?) */
   /*  D6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter I with Acute Accent */
   /*  D7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter I with Circumflex Accent */
   /*  D8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter I with Diaeresis */
   /*  D9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxul */
   /*  DA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* boxdr */
   /*  DB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Full block */
   /*  DC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Lower Half Block */
   /*  DD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Broken Bar */
   /*  DE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter I with Grave Accent */
   /*  DF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Upper Half Block */
   /*  E0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Acute Accent */
   /*  E1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small German Letter Sharp s */
   /*  E2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Circumflex Accent */
   /*  E3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Grave Accent */
   /*  E4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter o with Tilde */
   /*  E5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter O with Tilde */
   /*  E6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Micro Sign */
   /*  E7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Icelandic Eth */
   /*  E8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Icelandic Letter Eth */
   /*  E9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter U with Acute Accent */
   /*  EA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter U with Circumflex Accent */
   /*  EB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter U with Grave Accent */
   /*  EC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small Letter y with Acute Accent */
   /*  ED    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital Letter Y with Acute Accent */
   /*  EE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Macron */
   /*  EF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Acute Accent */
   /*  F0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* SHY (Syllable Hyphen)  */
   /*  F1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Plus-Minus Sign */
   /*  F2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Double Underscore */
   /*  F3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar Fraction Three Quarters */
   /*  F4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pilcrow Sign */
   /*  F5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Paragraph Sign, Section Sign */
   /*  F6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Division Sign */
   /*  F7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cedilla */
   /*  F8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Ring Above, Degree Sign */
   /*  F9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Diaeresis */
   /*  FA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Middle Dot */
   /*  FB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript One */
   /*  FC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript Three */
   /*  FD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript Two */
   /*  FE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Black Square */
   /*  FF     */  { CH_USED      ,  0xFF        ,  0xFF        }   /* NBSP (Required Blank) */
   };

   /*-------------------------------------------------------------+
   | ISO Registration Number 109//CHARSET                         |
   |     ECMA-94 Right Part of Latin Alphabet Nr. 3               |
   +-------------------------------------------------------------*/
#if 0  /* NOT YET IMPLEMENTED */
   static CHRDSC ANSI_X3_4_1986_chtab[MAX_CHARSET] = {
   };
#endif

   /*-------------------------------------------------------------+
   | ISO Registration Number 100//CHARSET                         |
   |     ECMA-94 Right Part of Latin Alphabet Nr. 1               |
   +-------------------------------------------------------------*/
   static CHRDSC ISO_100_ECMA_chtab[MAX_CHARSET] = {
   /*  80 N/A */  { CH_UNUSED    ,  0x80        ,  0x80        },
   /*  81 N/A */  { CH_UNUSED    ,  0x81        ,  0x81        },
   /*  82 N/A */  { CH_UNUSED    ,  0x82        ,  0x82        },
   /*  83 N/A */  { CH_UNUSED    ,  0x83        ,  0x83        },
   /*  84 N/A */  { CH_UNUSED    ,  0x84        ,  0x84        },
   /*  85 N/A */  { CH_UNUSED    ,  0x85        ,  0x85        },
   /*  86 N/A */  { CH_UNUSED    ,  0x86        ,  0x86        },
   /*  87 N/A */  { CH_UNUSED    ,  0x87        ,  0x87        },
   /*  88 N/A */  { CH_UNUSED    ,  0x88        ,  0x88        },
   /*  89 N/A */  { CH_UNUSED    ,  0x89        ,  0x89        },
   /*  8A N/A */  { CH_UNUSED    ,  0x8A        ,  0x8A        },
   /*  8B N/A */  { CH_UNUSED    ,  0x8B        ,  0x8B        },
   /*  8C N/A */  { CH_UNUSED    ,  0x8C        ,  0x8C        },
   /*  8D N/A */  { CH_UNUSED    ,  0x8D        ,  0x8D        },
   /*  8E N/A */  { CH_UNUSED    ,  0x8E        ,  0x8E        },
   /*  8F N/A */  { CH_UNUSED    ,  0x8F        ,  0x8F        },
   /*  90 N/A */  { CH_UNUSED    ,  0x90        ,  0x90        },
   /*  91 N/A */  { CH_UNUSED    ,  0x91        ,  0x91        },
   /*  92 N/A */  { CH_UNUSED    ,  0x92        ,  0x92        },
   /*  93 N/A */  { CH_UNUSED    ,  0x93        ,  0x93        },
   /*  94 N/A */  { CH_UNUSED    ,  0x94        ,  0x94        },
   /*  95 N/A */  { CH_UNUSED    ,  0x95        ,  0x95        },
   /*  96 N/A */  { CH_UNUSED    ,  0x96        ,  0x96        },
   /*  97 N/A */  { CH_UNUSED    ,  0x97        ,  0x97        },
   /*  98 N/A */  { CH_UNUSED    ,  0x98        ,  0x98        },
   /*  99 N/A */  { CH_UNUSED    ,  0x99        ,  0x99        },
   /*  9A N/A */  { CH_UNUSED    ,  0x9A        ,  0x9A        },
   /*  9B N/A */  { CH_UNUSED    ,  0x9B        ,  0x9B        },
   /*  9C N/A */  { CH_UNUSED    ,  0x9C        ,  0x9C        },
   /*  9D N/A */  { CH_UNUSED    ,  0x9D        ,  0x9D        },
   /*  9E N/A */  { CH_UNUSED    ,  0x9E        ,  0x9E        },
   /*  9F N/A */  { CH_UNUSED    ,  0x9F        ,  0x9F        },
   /*  A0    */  { CH_USED      ,  0xFF        ,  0xFF        },  /* NBSP: Required blank ??? */
   /*  A1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted exclamation mark */
   /*  A2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cent sign */
   /*  A3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pound sign */
   /*  A4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Currency sign */
   /*  A5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Yen sign */
   /*  A6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Broken bar */
   /*  A7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Paragraph Sign, section Sign */
   /*  A8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Diaeresis */
   /*  A9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Copyright sign */
   /*  AA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Feminine ordinal indicator */
   /*  AB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Left angle quotation mark */
   /*  AC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Not sign */
   /*  AD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* SHY: Soft hyphen ??? */
   /*  AE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Registered Trade Mark sign */
   /*  AF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Macron */
   /*  B0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Ring above, degree sign */
   /*  B1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Plus-minus sign
   /*  B2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 2 */
   /*  B3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 3 */
   /*  B4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Acute accent */
   /*  B5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Micro sign */
   /*  B6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Pilcrow sign */
   /*  B7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Middle dot */
   /*  B8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Cedilla */
   /*  B9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Superscript 1 */
   /*  BA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Masculine ordinal indicator */
   /*  BB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Right angle quotation mark */
   /*  BC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one quarter */
   /*  BD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction one half */
   /*  BE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Vulgar fraction three quarters */
   /*  BF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Inverted question mark */
   /*  C0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with grave accent */
   /*  C1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with acute accent */
   /*  C2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with circumflex accent */
   /*  C3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with tilde */
   /*  C4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with diaeresis */
   /*  C5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter A with ring above */
   /*  C6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital diphtong A with E */
   /*  C7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter C with cedilla */
   /*  C8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with grave accent */
   /*  C9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with acute accent */
   /*  CA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with circumflex accent */
   /*  CB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter E with diaeresis */
   /*  CC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with grave accent */
   /*  CD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with acute accent */
   /*  CE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with circumflex accent */
   /*  CF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter I with diaeresis */
   /*  D0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital icelandic letter ETH */
   /*  D1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter N with tilde */
   /*  D2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with grave accent */
   /*  D3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with acute accent */
   /*  D4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with circumflex accent */
   /*  D5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with tilde */
   /*  D6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with diaeresis */
   /*  D7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Multiplication sign */
   /*  D8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter O with oblique stroke */
   /*  D9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with grave accent */
   /*  DA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with acute accent */
   /*  DB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with circumflex accent */
   /*  DC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter U with diaeresis */
   /*  DD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital letter Y with acute accent */
   /*  DE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Capital icelandic letter THORN */
   /*  DF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small german letter sharp S */
   /*  E0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with grave accent */
   /*  E1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with acute accent */
   /*  E2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with circumflex accent */
   /*  E3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with tilde */
   /*  E4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with diaeresis */
   /*  E5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter A with ring above */
   /*  E6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small diphtong A with E */
   /*  E7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter C with cedilla */
   /*  E8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with grave accent */
   /*  E9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with acute accent */
   /*  EA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with circumflex accent *
   /*  EB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter E with diaeresis */
   /*  EC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with grave accent */
   /*  ED    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with acute accent */
   /*  EE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with circumflex accent *
   /*  EF    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter I with diaeresis */
   /*  F0    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small icelandic letter ETH */
   /*  F1    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter N with tilde */
   /*  F2    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with grave accent */
   /*  F3    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with acute accent */
   /*  F4    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with circumflex accent *
   /*  F5    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with tilde */
   /*  F6    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with diaeresis */
   /*  F7    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Division sign */
   /*  F8    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter O with oblique stroke */
   /*  F9    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with grave accent */
   /*  FA    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with acute accent */
   /*  FB    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with circumflex accent *
   /*  FC    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter U with diaeresis */
   /*  FD    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small letter Y with acute accent */
   /*  FE    */  { CH_USED      ,  (uchar)''  ,  (uchar)''  },  /* Small icelandic letter THORN */
   /*  FF     */  { CH_USED      ,  (uchar)''  ,  (uchar)''  }   /* Small letter Y with diaeresis */
   };

   switch(ahlid) {
   case ISO_646_1983_AHLID:     /* ISO 646-1983 (IRV) */
      return(ISO_646_1983_chtab);
   case ISO_8859_AHLID:         /* ISO 8859-1 Latin alphabet No.1 */
      return(ISO_8859_chtab);
   case IBM_395_AHLID:          /* IBM codepage 395 */
      return(IBM_395_chtab);
   case IBM_850_AHLID:          /* IBM codepage 850 */
      return(IBM_850_chtab);
   case ISO_100_ECMA_AHLID:     /* ISO registration 100 ECMA Nr. 1 */
      return(ISO_100_ECMA_chtab);
   case ANSI_X3_4_1986_AHLID:   /* ANSI X3.4-1986 (ASCII) */
      break;                    /* ANSI_X3_4_1986_chtab isn't implemented */
   default:
      break;
   }
   return(CHRDSC *)0;              /* Not found */
}
/******************************************************************************/
